// SHS mode set
// SHSMode.c
// osamu imaizumi
// 20-Feb-97 V1

#include <windows.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <winioctl.h>
#include "shsioctl.h"        // This defines the IOCTL constants.

/*#define DIVISOR*/

void __cdecl main(int argc, char **argv)
{

    // The following is returned by IOCTL.  It is true if the write succeeds.
    BOOL    IoctlResult = TRUE;

    // The following parameters are used in the IOCTL call
    HANDLE               hndFile;        // Handle to device, obtain from CreateFile
    SERIAL_SET_FIXED_DIVISOR DivisorBuffer;  // buffer for DeviceIoControl
    SERIAL_SMC_PRESENT   InputBuffer;    // Input buffer for DeviceIoControl
    SERIAL_SMC_PRESENT   DataBuffer;     // data buffer for DeviceIoControl
    LONG                 IoctlCode;
    ULONG                DataLength;
    ULONG                ReturnedLength; // Number of bytes returned in output buffer

    int                 port;
    char                *p,devName[32];

#ifdef DIVISOR
    printf("SHSMode V1.0D\n");
    if (argc < 2 || argc > 4) {
        printf("usage:SHSMode <Port#> [<Value>]\n");
        printf("  or  SHSMode <Port#> <Divisor(hex)> <Baudrate>\n\n");
#else
    printf("SHSMode V1.0\n");
    if (argc < 2 || argc > 3) {
        printf("usage:SHSMode <Port#> [<Value>]\n\n");
#endif
        exit(1);
    }

    sscanf(argv[1],"%d",&port);
    sprintf(&devName[0],"\\\\.\\com%d",port);
    DataLength = sizeof(DataBuffer.SmcPresent);

    hndFile = CreateFile(
                    devName,           // Open the Device "file"
                    GENERIC_WRITE,
                    FILE_SHARE_WRITE,
                    NULL,
                    OPEN_EXISTING,
                    0,
                    NULL);

    if (hndFile == INVALID_HANDLE_VALUE) {      // Was the device opened?
        printf("Unable to open the device -'com%d'.\n", port);
        switch(GetLastError()) {
        case 5:
            printf("Device in use.\n");
            break;
        case 2:
            printf("Device not exist.\n" );
            break;
        default:
            printf("Device open failed with code %ld\n", GetLastError() );
            break;
        }
        exit(1);
    }

    switch (argc) {
#ifdef DIVISOR
    case 4:
        IoctlCode = IOCTL_SERIAL_SET_FIXED_DIVISOR;
        sscanf(argv[2], "%x", &DivisorBuffer.FixedDivisor); // Get the value to be set.
        sscanf(argv[3], "%ld", &DivisorBuffer.CurrentBaud); // Get the value to be set.
       
        IoctlResult = DeviceIoControl(
                        hndFile,                // Handle to device
                        IoctlCode,              // IO Control code for Write
                        &DivisorBuffer,         // Buffer to driver.  Holds port & data.
                        sizeof(DivisorBuffer),  // Length of buffer in bytes.
                        NULL,                   // Buffer from driver.   Not used.
                        0,                      // Length of buffer in bytes.
                        &ReturnedLength,        // Bytes placed in outbuf.  Should be 0.
                        NULL                    // NULL means wait till I/O completes.
                        );
        if (IoctlResult)                        // Did the IOCTL succeed?
            printf("com%d: FixedDivisor = %08xh, CurrentBaud = %7d\n",
                    port,DivisorBuffer.FixedDivisor,DivisorBuffer.CurrentBaud);
        break;
#endif
    case 3:
        IoctlCode = IOCTL_SERIAL_SET_SMC_PRESENT;
        sscanf(argv[2], "%x", &InputBuffer.SmcPresent); // Get the value to be set.
        
        IoctlResult = DeviceIoControl(
                        hndFile,                // Handle to device
                        IoctlCode,              // IO Control code for Write
                        &InputBuffer,           // Buffer to driver.  Holds port & data.
                        sizeof(InputBuffer),    // Length of buffer in bytes.
                        NULL,                   // Buffer from driver.   Not used.
                        0,                      // Length of buffer in bytes.
                        &ReturnedLength,        // Bytes placed in outbuf.  Should be 0.
                        NULL                    // NULL means wait till I/O completes.
                        );
        /* no break; */
    case 2:
        if (IoctlResult) {                      // Did the IOCTL succeed?
            IoctlCode = IOCTL_SERIAL_GET_SMC_PRESENT;
            IoctlResult = DeviceIoControl(
                          hndFile,          // Handle to device
                          IoctlCode,        // IO Control code for Read
                          NULL,             // Buffer to driver.
                          0,                // Length of buffer in bytes.
                          &DataBuffer,      // Buffer from driver.
                          DataLength,       // Length of buffer in bytes.
                          &ReturnedLength,  // Bytes placed in DataBuffer.
                          NULL              // NULL means wait till op. completes.
                          );
        }
        if (IoctlResult) { // Did the IOCTL succeed?
            switch(DataBuffer.SmcPresent& 0xff80){
            case 0x4080: p = "67x"; break;
            case 0x4880: p = "68x"; break;
            case 0x0280: p = "93x"; break;
            case 0x3080: p = "93xAPM"; break;
            case 0x0380: p = "93xFR"; break;
            case 0x0780: p = "957FR"; break;
            case 0x0300: p = "669"; break;
            case 0x0400: p = "669FR"; break;
            case 0x6500: p = "665IR"; break;
            case 0x6600: p = "666IR"; break;
            default:
                DataBuffer.SmcPresent = 0;
                p = "UNKNOWN"; // never used.
                break;
            }
            if (DataBuffer.SmcPresent) {
                printf("com%d: SMC37C%s at %04xh, Mode = %02xh\n",
                    port,p,DataBuffer.SmcPresent >> 16,DataBuffer.SmcPresent & 0x7f);
            } else
                printf("com%d is not SMC serial port.\n",port);
        }
    default:
        break;
    }
    if (!IoctlResult) {
        if (GetLastError() == 87)
            printf("*Error* - No SHS driver installed.\n");
        else
            printf("Ioctl failed with code %ld\n", GetLastError() );
        exit(1);
    }
    if (!CloseHandle(hndFile))                  // Close the Device "file".
    {
        printf("Failed to close device.\n");
    }   
    exit(0);
}
